//+------------------------------------------------------------------+
//|                                                        ct_17.mq5 |
//|                                  Copyright 2022, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2022, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\SignalRSI.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingNone.mqh>
//--- available money management
#include <Expert\Money\My\MoneyCT_17.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string             Expert_Title                                    = "ct_17";    // Document name
ulong                    Expert_MagicNumber                              = 22361;      //
bool                     Expert_EveryTick                                = false;      //
//--- inputs for main signal
input int                Signal_ThresholdOpen                            = 10;         // Signal threshold value to open [0...100]
input int                Signal_ThresholdClose                           = 10;         // Signal threshold value to close [0...100]
input double             Signal_PriceLevel                               = 0.0;        // Price level to execute a deal
input double             Signal_StopLevel                                = 50.0;       // Stop Loss level (in points)
input double             Signal_TakeLevel                                = 50.0;       // Take Profit level (in points)
input int                Signal_Expiration                               = 4;          // Expiration of pending orders (in bars)
input int                Signal_RSI_PeriodRSI                            = 8;          // Relative Strength Index(8,...) Period of calculation
input ENUM_APPLIED_PRICE Signal_RSI_Applied                              = PRICE_CLOSE;// Relative Strength Index(8,...) Prices series
input double             Signal_RSI_Weight                               = 1.0;        // Relative Strength Index(8,...) Weight [0...1.0]
//--- inputs for money
input double             Money_DiscriminantAnalysis_DecreaseFactor       = 3.0;        // Decrease factor
input double             Money_DiscriminantAnalysis_Percent              = 12.5;       // Percent
input int                Money_DiscriminantAnalysis_TimeframeOperation   = 0;          // Timeframe Operation
input int                Money_DiscriminantAnalysis_TimeframeIdentity    = 0;          // Timeframe Identity
input int                Money_DiscriminantAnalysis_LookbackOperation    = 0;          // Lookback Operation
input int                Money_DiscriminantAnalysis_LookbackIdentity     = 0;          // Lookback Identity
input int                Money_DiscriminantAnalysis_AppliedpriceOperation= 0;          // Appliedprice Operation
input int                Money_DiscriminantAnalysis_AppliedpriceIdentity = 0;          // Appliedprice Identity
input int                Money_DiscriminantAnalysis_IndicatorOperation   = 0;          // Indicator Operation
input int                Money_DiscriminantAnalysis_IndicatorIdentity    = 0;          // Indicator Identity
input bool               Money_DiscriminantAnalysis_Objects              = true;       // Use Objects to Objects
input double             Money_DiscriminantAnalysis_Decay                = 0.001;      // Multi Layer Perceptron Decay
input int                Money_DiscriminantAnalysis_Restarts             = 2;          // Multi Layer Perceptron Restarts
input int                Money_DiscriminantAnalysis_Hidden               = 7;          // Hidden Layer Size
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
{
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(), Period(), Expert_EveryTick, Expert_MagicNumber))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing expert");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Creating signal
   CExpertSignal *signal = new CExpertSignal;
   if(signal == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating signal");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalRSI
   CSignalRSI *filter0 = new CSignalRSI;
   if(filter0 == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating filter0");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.PeriodRSI(Signal_RSI_PeriodRSI);
   filter0.Applied(Signal_RSI_Applied);
   filter0.Weight(Signal_RSI_Weight);
//--- Creation of trailing object
   CTrailingNone *trailing = new CTrailingNone;
   if(trailing == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Set trailing parameters
//--- Creation of money object
   CMoneyCT *money = new CMoneyCT;
   if(money == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Set money parameters
   money.DecreaseFactor(Money_DiscriminantAnalysis_DecreaseFactor);
   money.Percent(Money_DiscriminantAnalysis_Percent);
   money.TimeframeOperation(Money_DiscriminantAnalysis_TimeframeOperation);
   money.TimeframeIdentity(Money_DiscriminantAnalysis_TimeframeIdentity);
   money.LookbackOperation(Money_DiscriminantAnalysis_LookbackOperation);
   money.LookbackIdentity(Money_DiscriminantAnalysis_LookbackIdentity);
   money.AppliedpriceOperation(Money_DiscriminantAnalysis_AppliedpriceOperation);
   money.AppliedpriceIdentity(Money_DiscriminantAnalysis_AppliedpriceIdentity);
   money.IndicatorOperation(Money_DiscriminantAnalysis_IndicatorOperation);
   money.IndicatorIdentity(Money_DiscriminantAnalysis_IndicatorIdentity);
   money.Objects(Money_DiscriminantAnalysis_Objects);
   money.Decay(Money_DiscriminantAnalysis_Decay);
   money.Restarts(Money_DiscriminantAnalysis_Restarts);
   money.Hidden(Money_DiscriminantAnalysis_Hidden);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
   {  //--- failed
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing indicators");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- ok
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{  ExtExpert.Deinit();
}
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
{  ExtExpert.OnTick();
}
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
{  ExtExpert.OnTrade();
}
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
{  ExtExpert.OnTimer();
}
//+------------------------------------------------------------------+
